var FleksyAPI;

var fleksy = new Fleksy({
	element: '#fleksy',
	aspect: $(window).width() > 700 ? 2.25 : 1.6,
	width: $(window).width(),
	imagePath: ''
});

function setPlatform( name ) {
	if ( name == "Android" ) {
		FleksyAPI = new FleksyAPIAndroid();
	} else if ( name == 'iOS' ) {
		FleksyAPI = new FleksyAPIiOS();
	} else {
		//console.log("Undefined Platform!");
	}

	FleksyAPI.requestStoreData();
}

function callSaveUserThemeCallback() {
	ThemeBuilder.saveTheme();
}

function userThemesCallback(themes, imagePath) {
	var userThemes;
	if ( typeof themes == 'string' ) {
		userThemes = $.parseJSON( themes );
	} else {
		userThemes = themes;
	}

	ThemeBuilder.globals.imagePath = imagePath;
	ThemeBuilder.globals.userThemes = userThemes;
	ThemeBuilder.handleAfterThemeSave();
}

function purchasedCallback( data ) {
  if ( typeof data == 'string' ) {
		ThemeBuilder.globals.purchases = $.parseJSON(data).purchases;
  } else {
		ThemeBuilder.globals.purchases = data.purchases;
  }
}

function inventoryCallback( data ) {
	if ( typeof data == 'string' ) {
		ThemeBuilder.globals.inventory = $.parseJSON(data).inventory;
	} else {
		ThemeBuilder.globals.inventory = data.inventory;
	}
}

function purchaseResultCallback(id, result) {
	if ( result === 'success' || result === '200 OK' ) {
		ThemeBuilder.globals.purchases.push(id);
	} else {
		//FleksyAPI.log( result );
	}
}

function selectedUserThemeCallback( themeData, imagePath ) {
	var themeDataJSON = typeof themeData === 'string' ? $.parseJSON( themeData ) : themeData ;
	var themeObj = typeof themeDataJSON.key === 'undefined' ? fleksy.theme() : themeDataJSON;

	if ( typeof themeDataJSON.key !== 'undefined' && themeObj.background && themeObj.background.gradient ) {
		//TODO make theme application straightforward.
		fleksy.theme(themeObj);
		fleksy.clearBackground();
		_.each( themeObj.background.gradient, function(color, idx) {
			fleksy.backgroundColor(color, idx);
		});

		ThemeBuilder.init(themeObj);

		if ( typeof themeObj.background.image !== 'undefined' ) {
			fleksy.backgroundImagePreview(imagePath + '/' + themeObj.background.image);
			ThemeBuilder.styleThemeIconImage(imagePath + '/' + themeObj.background.image);
		}
	} else {
		ThemeBuilder.init(themeObj);
	}
}

function imagePickedCallback(image) {
	fleksy.backgroundImage(image);
	ThemeBuilder.styleThemeIconImage(image);
	ThemeBuilder.toggleBackgroundImageRemoveButtonDisplay();
}

var ThemeBuilder = {
	globals: {
		isBuilderPurchased: false,
		inventory: null,
		purchases: null,
		userThemes: {},
		dragend: '',
		currentThemeKey: '',
		imagePath: '',
		recentsPalette: []
	},

	init: function(theme) {
		var renderData = {
			theme: theme,
			tinycolor: tinycolor,
			icons: ['', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '']
		};

		var backgroundImageView = new EJS({ url: 'templates/partials/background_image.ejs?v=' + Date.now() }).render(renderData),
				backgroundColorView = new EJS({ url: 'templates/partials/background_color.ejs?v=' + Date.now() }).render(renderData),
				lettersView = new EJS({ url: 'templates/partials/letters.ejs?v=' + Date.now() }).render(renderData),
				buttonsView = new EJS({ url: 'templates/partials/buttons.ejs?v=' + Date.now() }).render(renderData),
				saveView = new EJS({ url: 'templates/partials/save.ejs?v=' + Date.now()  }).render(renderData),
				view = backgroundImageView + backgroundColorView + lettersView + buttonsView + saveView;

		$('.pagedrag').append(view);

		ThemeBuilder.globals.dragend = new Dragend($("#wrap").get(0), {
			afterInitialize: function () {
				$("#wrap").css("visibility", "visible");
				$('.dragend-page').css('padding-bottom', $("#fleksy").height());
				ThemeBuilder.bindThemeBuilderUI();
			},

			onSwipeEnd: function() {
				var $activePage = $(this.activeElement),
						$navigation = $('.navigation'),
						$navs = $navigation.find('li');

				$navs.removeClass('nav-active');
				$navs.eq($activePage.index()).addClass('nav-active');

				if ( $activePage.index() === 2 ) {
					fleksy.togglePop(true);
				} else {
					fleksy.togglePop(false);
				}
			}
		});

		$('.save-theme').on('click', function(e) {
			e.preventDefault();
			ThemeBuilder.saveTheme();
		});
	},

	bindThemeBuilderUI: function() {
		this.toggleBackgroundImageRemoveButtonDisplay();
		this.bindColorPickers();
		this.bindBackgroundColorInputs();
		this.bindImageOpacitySlider();
		this.bindBackgroundImageButton();
		this.bindBackgroundImagePosition();
		this.bindDropdowns();
		this.bindBuilderPurchaseButton();
		$('.control-picker').spectrum('option', 'palette', this.setPresetColors());
	},

	bindColorPickers: function() {
		var _this = this;

		$(".control-picker").not('.background-template').spectrum({
			showAlpha: true,
			preferredFormat: "hex3",
			clickoutFiresChange: true,
			chooseText: "Close",
			cancelText: "",
			showPalette: true,
			palette: _this.globals.recentsPalette,
			move: function(color) {
				var that = $(this);
				var rgbaColor = color.toRgbString();
				var action = that.data('action');

				if ( typeof fleksy[action] !== 'undefined' ) {
					fleksy[action](color.toHex8String(), that.parent().hasClass('control-wrap') ? that.parent().index() : that.index());
					that.find('.control-color').css({
						backgroundColor: rgbaColor
					});

					// When a user edits key color we style candyFocus and candyColor
					//  with the same color as well
					if ( action === 'keyColor' ) {
						var $control = $('.control');
						fleksy.candyFocusColor(color.toHex8String());
						fleksy.candyColor(color.setAlpha(.6).toHex8String());

						$control.filter(function() {
							return $(this).data('action') === 'candyFocusColor';
						}).find('.control-color').css({
							backgroundColor: rgbaColor
						});

						$control.filter(function() {
							return $(this).data('action') === 'candyColor';
						}).find('.control-color').css({
							backgroundColor: color.setAlpha(.6).toRgbString()
						});
					}
				}
			},
			beforeShow: function(color) {
				var $this = $(this);
				var windowHeight = $(window).height();
				var keyboardHeight = $('#fleksy').height();
				var $spContainer = $('.sp-container');
				var $spPickerContainer = $('.sp-picker-container');

				$spContainer.height(windowHeight - keyboardHeight);
				$spPickerContainer.height($spContainer.height() - (_this.globals.recentsPalette.length ? 31 : 0));
				$this.spectrum('set', $this.find('.control-color').css('background-color'));
			},
			hide: function(color) {
				_this.addColorToRecentPalette(color.toHexString());
			}
		});
	},

	addColorToRecentPalette: function(color) {
		var $colorPickers = $('.control-picker');
		var pickersPalette = this.globals.recentsPalette;

		if ( !~pickersPalette.indexOf(color) ) {
			if ( pickersPalette.length > 5 ) pickersPalette.pop();

			pickersPalette.unshift(color);
			$colorPickers.spectrum('option', 'palette', pickersPalette);
		}
	},

	setPresetColors: function() {
		var theme = fleksy.theme();
		var pickersPalette = this.globals.recentsPalette;

		if ( theme.background && theme.background.gradient ) {
			theme.background.gradient.forEach(function(color) {
				pickersPalette.push(color);
			});
		}

		pickersPalette.push(theme.colors.letters);
		pickersPalette.push(theme.colors.homerow);
		pickersPalette.push(theme.colors.inner_btn);

		return pickersPalette;
	},

	bindImageOpacitySlider: function() {
		var opacitySlider = document.getElementById('bg-image-opacity-slider');

		noUiSlider.create(opacitySlider, {
			start: opacitySlider.dataset.start,
			step: 0.1,
			range: {
				min: 0,
				max: 1
			}
		});

		opacitySlider.noUiSlider.on('update', function(values, handle) {
			fleksy.backgroundImageOpacity(values[0]);
		});
	},

	bindBackgroundColorInputs: function() {
		var $backgroundWrapper = $('.background-stops');
		var $addBtn = $('.add-background-color');

		$backgroundWrapper.on('click', '.control-remove', function(e) {
			e.preventDefault();
			var $parent = $(this).parent();

			fleksy.backgroundColor(null, $parent.index());
			$parent.fadeOut('fast', function() {
				$parent.remove();
				if ( !$addBtn.is(':visible') && ThemeBuilder.getBackgroundColorInputsNumber() < 6 ) {
					$addBtn.fadeIn('fast');
				}
			});
		});

		// Background colors
		$addBtn.on('click', function(e) {
			e.preventDefault();

			$backgroundWrapper
				.find('.control-wrap')
				.last()
				.clone(true)
				.appendTo($backgroundWrapper);

			if ( ThemeBuilder.getBackgroundColorInputsNumber() > 5 ) {
				$('.add-background-color').fadeOut('fast');
			}

			ThemeBuilder.bindColorPickers();
		});
	},

	getBackgroundColorInputsNumber: function() {
		return $('.background-stops').find('.control-wrap').length;
	},

	setSaveScreenState: function() {
		var $themeSaveWrap = $('.theme-save-callout');

		$themeSaveWrap.find('.theme-callout-title-save').show();
		if ( ThemeBuilder.isThemeBuilderPurchased() ) {
			$themeSaveWrap.find('.builder-not-purchased').hide();
			$themeSaveWrap.find('.builder-purchased').show();
		} else {
			$themeSaveWrap.find('.builder-purchased').hide();
			$themeSaveWrap.find('.builder-not-purchased').show();
		}
	},

	/**
	* Creates an RFC4122 v4 compliant UUID
	* http://stackoverflow.com/a/2117523/4626015
	*
	* @return {String} UUID
	*/
	createUUID: function() {
		return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
			var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
			return v.toString(16);
		});
	},

	isThemeSaved: function(themeKey) {
		return typeof ThemeBuilder.globals.userThemes[themeKey] !== 'undefined';
	},

	saveTheme: function() {
		var theme_id = $("#theme_id").data('theme_id');
		var theme = fleksy.theme();
		theme.key = theme_id.length ? theme_id : 'usertheme_' + ThemeBuilder.createUUID();
		theme.name = 'Fleksy User Theme';
		ThemeBuilder.globals.currentThemeKey = theme.key;

		if ( typeof theme.background.image !== 'undefined' ) {
			theme.background.image = 'image_' + theme.key + '.png';
			ThemeBuilder.styleThemeIconImage(ThemeBuilder.globals.imagePath + '/' + theme.background.image);
		}

		FleksyAPI.saveUserTheme(JSON.stringify(theme), theme.key);
	},

	handleAfterThemeSave: function() {
		if ( ThemeBuilder.globals.currentThemeKey !== '' ) { // need this to prevent firing on themebuilder load
			if ( ThemeBuilder.isThemeSaved(ThemeBuilder.globals.currentThemeKey) ) {
				ThemeBuilder.styleThemeIcon(fleksy.theme());
				$('.save-theme').remove();
				$('.top-bar').remove();
				$('#fleksy').hide();
				$('.dragend-page').not(':last-child').remove();
				ThemeBuilder.globals.dragend.destroy();
				$("#wrap").css('visibility', 'visible');
				$('.theme-save-callout').fadeIn('fast');
				$('.share-theme').on('click', function(e) {
					e.preventDefault();
					FleksyAPI.shareTheme(ThemeBuilder.globals.currentThemeKey);
				});
				FleksyAPI.showToast('Theme saved!');
			} else {
				FleksyAPI.showToast('Saved failed. Please try again.');
			}
		}
	},

	bindBackgroundImageButton: function() {
		$('.control-bg-image').on('click', function(e) {
			e.preventDefault();
			FleksyAPI.pickImage();
		});

		$('.bg-image-remove').on('click', function(e) {
			e.preventDefault();
			fleksy.backgroundImage();
			ThemeBuilder.styleThemeIconImage('');
			ThemeBuilder.toggleBackgroundImageRemoveButtonDisplay();
		});

		$('.bg-image-edit').on('click', function(e) {
			e.preventDefault();
			FleksyAPI.editThemeImage(fleksy.theme().background.image.split('/').pop());
		});
	},

	toggleBackgroundImageRemoveButtonDisplay: function() {
		var $btn = $('.bg-image-remove, .bg-image-edit');
		if ( typeof fleksy.theme().background.image !== 'undefined' ) {
			$btn.fadeIn('fast');
		} else {
			$btn.fadeOut('fast');
		}
	},

	bindBackgroundImagePosition: function() {
		var $imgPositionBtn = $('.bg-image-position');

		$imgPositionBtn.on('click', function(e) {
			e.preventDefault();
			if ( $(this).hasClass('center') ) {
				fleksy.backgroundImagePosition('center');
			} else {
				fleksy.backgroundImagePosition('scale');
			}

			$imgPositionBtn.addClass('inactive');
			$(this).removeClass('inactive');
		});
	},

	bindDropdowns: function() {
		// Key Pop Icon
		$('.control-dropdown').on('click', function(e) {
			if ( $(e.target).parent().hasClass('pop-icons') || $(e.target).hasClass('dropdown-select-close') ) {
				return;
			}

			e.preventDefault();

			$(this).find('.dropdown-select').fadeIn('fast');
		});

		var $dropdownSelect = $('.dropdown-select');
		$dropdownSelect.on('click', 'span', function(e) {
			e.preventDefault();
			var $that = $(this);
			$dropdownSelect.find('span').removeClass('active');
			$that.addClass('active');

			var $parentDropdown = $that.parents('.control-dropdown');

			var action = $parentDropdown.data('action');
			$parentDropdown.find('.control-icon').text($that.text());
			fleksy[action]($that.text());
			$that.parents('.dropdown-select').fadeOut('fast');
		});

		$('.dropdown-select-close').on('click', function(e) {
			e.preventDefault();
			$(this).parent().fadeOut('fast');
		});
	},

	styleThemeIcon: function(themeObj) {
		var theme = $('.theme-preview');
		var themeColors = ( themeObj.background ) ? themeObj.background.gradient : '';

		if ( themeColors ) {
			var colorStops = '';
			var stop;
			var colorNum = themeColors.length;

			if ( colorNum == 1 ) {
				colorStops = tinycolor( themeColors[0] ).toHexString() + " 0%, " + tinycolor( themeColors[0] ).toHexString() + " 100%";
			} else {
				_.each( themeColors, function( color, index ) {
					if ( index == 0 ) {
						stop = 7;
						colorStops += tinycolor( color ).toHexString() + " " + stop +  "%, ";
					}	else if ( index == colorNum - 1 ) {
						stop = 100;
						colorStops += tinycolor( color ).toHexString() + " " + stop +  "%";
					} else {
						stop = (100 / (colorNum - 1))*index + 7;
						colorStops += tinycolor( color ).toHexString() + " " + stop +  "%, ";
					}
				});
			}

			theme.css({
				"background-image": "-webkit-linear-gradient(top, " + colorStops + ")"
			});
		}
	},

	styleThemeIconImage: function(image) {
		$('.theme-preview-image').css({
			'background-image': 'url(' + image +')'
		});
	},

	/**
	* Gets the Theme Builder's IAP key from the inventory
	*
  * @param {Object} inventory
 	* @return {String} themebuilder key
	*/
	getThemeBuilderIAPKey: function(inventory) {
		var key = '';

		_.each(inventory, function(item) {
			if ( ~item.key.indexOf('themebuilder') ) {
				key = item.key;
			}
		});

		return key;
	},

	/**
	* Checks if the Theme Builder is purchased or not
	*
 	* @return {Number} 0 if not purchased, valid integer if purchased
	*/
	isThemeBuilderPurchased: function() {
		var themeBuilderKey = this.getThemeBuilderIAPKey(this.globals.inventory);

		return ~this.globals.purchases.indexOf(themeBuilderKey);
	},


	/**
	* Binds the theme builder purchase button
	*/
	bindBuilderPurchaseButton: function() {
		$('.builder-purchase').on('click', function(e) {
			e.preventDefault();

			FleksyAPI.purchaseBox(ThemeBuilder.getThemeBuilderIAPKey(ThemeBuilder.globals.inventory));
		});
	}
};