(function (root, factory) {
	if (typeof define === 'function' && define.amd) {
		define(['jquery, tinycolor'], factory);
	} else if (typeof module === 'object' && typeof module.exports === 'object') {
		module.exports = factory(require('jquery'), require('tinycolor'));
	} else {
		root.Fleksy = factory(root.jQuery, root.tinycolor);
	}
}(this, function ($, tinycolor) {

	var
	global = this,
	defaults = {
		'element': null,
		'width': 640,
		'height': null,
		'aspect': 1.423076923,
		'imagePath': '/',
		'theme': {
			"name": "Fleksy Theme",
			"key": "",
			"colors": {
				"homerow": "#30ffffff",
				"inner_btn": "#24ffffff",
				"press_btn": "#55000000",
				"shade_btn": "#5000000",
				"letters": "#E1E1E1",
				"tile": "#A6A3A3A3",
				"candy": "#80E1E1E1",
				"candyfocus": "#E1E1E1",
				"swipe": "#1DFFFFFF",
				"outline": "#00FFFFFF"
			},
			"icon": {},
			"icons": {
				"tile": ""
			},
			"background": {
				"gradient": [
					"#000000"
				]
			}
		}
	},
	layouts = {
		'english': [
			'Q W E R T Y U I O P',
			'A S D F G H J K L',
			'{shift} Z X C V B N M {backspace}',
			'{num} {magic} {spacebar} {period} {enter}'
		],
		'korean': [
			'ㅂ ㅈ ㄷ ㄱ ㅅ ㅛ ㅕ ㅑ ㅐ ㅔ',
			'ㅁ ㄴ ㅇ ㄹ ㅎ ㅗ ㅓ ㅏ ㅣ ',
			'{shift} ㅋ ㅌ ㅊ ㅍ ㅠ ㅜ ㅡ {backspace}',
			'{num} {magic} {spacebar} {period} {enter}'
		],
		'japanese': [
			'あ か さ',
			'た な は',
			'ま や ら',
			'{shift} 小 わ {backspace}',
			'{num} {magic} {spacebar} {period} {enter}'
		]
	};

	function makeKey(keyStr) {
		var key = $('<span/>', {
									'class': 'fl-key'
								}).css({
									'display': 'table-cell',
									'vertical-align': 'middle',
									'position': 'relative',
									'width': '10%'
								});

		if ( /^\{\S+\}$/.test(keyStr) ) {
			var val = keyStr. match(/^\{(\S+)\}$/)[1];
			switch(val) {
				case 'shift':
					key
						.html('<i class="icon_shift_off"></i>')
						.css({
							'opacity': 0.7,
							'font-size': '0.85em',
							'line-height': '0'
						});
					break;
				case 'backspace':
					key
						.html('<i class="icon_backspace"></i>')
						.css({
							'opacity': 0.7,
							'font-size': '0.85em',
							'line-height': '0'
						});
					break;
				case 'num':
					key
						.addClass('fl-btn')
						.html('<i class="icon_numbers"></i>')
						.css({
							'line-height': '0',
							'padding': '0 1%',
							'font-size': '1.15em',
							'border-radius': '0.06em'
						});
					break;
				case 'magic':
					key
						.addClass('fl-btn')
						.html('<i class="icon_emoji"></i>')
						.css({
							'line-height': '0',
							'font-size': '1.15em',
							'border-radius': '0.06em'
						});
					break;
				case 'spacebar':
					key
						.addClass('fl-btn fl-spacebar')
						.text(' ')
						.css({
							'width': '48%',
							'line-height': '0',
							'border-radius': '0.06em'
						});
					break;
				case 'period':
					key
						.addClass('fl-btn')
						.html('<i class="icon_period_numpad"></i>')
						.css({
							'line-height': '0',
							'font-size': '1.15em',
							'border-radius': '0.06em'
						});
					break;
				case 'enter':
					key
						.addClass('fl-btn')
						.html('<i class="icon_enter"></i>')
						.css({
							'line-height': '0',
							'padding': '0 1%',
							'font-size': '1.15em',
							'border-radius': '0.06em'
						});
					break;
			}
		} else {
			var popIcon = $('<i/>', {
				'class': ''
			}).css({
				'position': 'absolute',
				'text-shadow': 'none',
				'top': 0,
				'left': 0,
				'font-size': '3.4em',
				'line-height': '0.3em',
				'z-index': -1,
				'font-family': 'flicon',
				'font-style': 'normal',
				'text-align': 'center'
			}).text('');

			var pop = $('<b/>', {
				'class': 'fl-key-pop'
			}).css({
				'opacity': 0,
				'pointer-events': 'none',
				'width': '100%',
				'height': '100%',
				'top': '0.6em',
				'left': 0,
				'font-weight': 'normal',
				'position': 'absolute',
				'transition': 'top 0.07s ease, opacity 0.18s ease',
				'line-height': 1,
				'text-align': 'center',
				'z-index': 10
			}).text(keyStr).append(popIcon);

			setTimeout(function() {
				var leftPx = (pop.outerWidth() - popIcon.outerWidth())/2,
						leftPerc = leftPx/pop.outerWidth()*100 + '%';

				popIcon.css({
					'left': leftPerc
				});
			}, 2000); //FIXME this is stupid and needs to be rethought

			var strWrap = $('<span/>').css({
				'font-style': 'normal',
				'transition': 'opacity 0.12s ease'
			}).text(keyStr);

			key.append(strWrap, pop);
		}

		return key;
	}

	function makeCandy() {
		var $candy = $('<div/>', {
			'class': 'fl-candy'
		}).css({
			'display': 'table',
			'text-align': 'center',
			'position': 'absolute',
			'top': '0.15em',
			'left': 0,
			'font-size': '0.47em',
			'font-weight': 300,
			'width': '100%',
			'line-height': 1
		});

		var arr = ['suggestion', 'autocorrect', 'suggestion'];

		for ( var i = 0; i < arr.length; i++ ) {
			var $candyEl = $('<span/>').css({
				'display': 'table-cell',
				'vertical-align': 'middle',
				'width': '33.3333%'
			}).text(arr[i]);

			i === 1 ? $candyEl.addClass('fl-candyfocus') : $candyEl.addClass('fl-candy');

			$candy.append($candyEl);
		}

		return $candy;
	}

	function Fleksy(opts) {
		var self = this;
		this.options = $.extend({}, defaults, opts);
		$(this.options.element).css({
			'font-family': 'Roboto, Helvetica, Arial, sans-serif',
			'text-align': 'center',
			'max-width': '100%',
			'position': 'relative',
			'z-index': 100,
			'user-select': 'none',
			'cursor': 'default'
		});

		this.width(this.options.width);
		this.layout(layouts.english);
		setTimeout(function() {
			fitToViewport.call(self);
		}, 0);

		if ( typeof global.onresize !== 'undefined' ) {
			$(global).on('resize', fitToViewport.bind(this));
		}
	}

	Fleksy.prototype.element = function() {
		return this.options.element;
	};

	Fleksy.prototype.layout = function(layout) {
		var $keyboard = $(this.options.element),
				rowNum = layout.length,
				$rows = [],
				that = this;

		$keyboard.html('');
		var $inner = $('<div/>', {'class': 'kb-inner'}).css({
			'position': 'absolute',
			'backgroundRepeat': 'no-repeat',
			'backgroundPosition': 'center',
			'z-index': -1,
			'top': 0,
			'bottom': 0,
			'left': 0,
			'right': 0
		}).appendTo($keyboard);

		$keyboard.append(makeCandy());

		layout.forEach(function(keys, index) {
			var keysNo = keys.split(' ').length;
			var $row = $('<div/>', {
				class: index !== 1 ? 'fl-row' : 'fl-row fl-row-mid'
			});

			keys.split(' ').forEach(function(key) {
				$row.append(makeKey(key, rowNum));
			});

			$row.css({
				'display': 'table',
				'width': '100%',
				'height': 100/rowNum + '%'
			});

			// add spacing in the button row
			if ( index == 3 ) {
				$row.css({
					'border-spacing': '0.09em'
				});
			}

			$rows.push($row);
		});

		$keyboard.append($rows);

		this.theme(this.options.theme);

		$('.fl-key')
			.on('mousedown', function(e) {
				e.preventDefault();
				var keyPop = $(this).find('b');
				$(this).find('span').css({
					'opacity': 0
				});

				keyPop
					.css({
						'opacity': 1,
						'top': '-18%'
					}).addClass('fl-toggled');
			})
			.on('mouseup', function(e) {
				e.preventDefault();
				var keyPop = $(this).find('b');
				$(this).find('span').css('opacity', 1);

				keyPop
					.css({
						'opacity': 0,
						'top': '0'
					}).removeClass('fl-toggled');
			});
	};

	Fleksy.prototype.aspect = function(aspect) {
		if ( typeof aspect !== 'undefined' ) {
			this.options.aspect = aspect;
			this.height(this.width()/aspect);
		}

		return this.options.aspect;
	};

	Fleksy.prototype.width = function(width) {
		if ( typeof width !== 'undefined' ) {
			$(this.options.element).width(width);
			this.options.width = width;

			if ( width/this.height() != this.aspect() ) {
				this.height(width/this.options.aspect);
			}
		}

		return this.options.width;
	};

	Fleksy.prototype.height = function(height) {
		if ( typeof height !== 'undefined' ) {
			$(this.options.element).height(height);
			this.options.height = height;
			this.setLetterSize();

			if ( this.width()/height != this.aspect() ) {
				this.width(height/this.options.aspect);
			}
		}

		return this.options.height;
	};

	Fleksy.prototype.setLetterSize = function() {
		$(this.options.element).css({
			'font-size': (this.height()/4)*0.451282051
		});
	};

	Fleksy.prototype.theme = function(theme) {
		if ( theme ) {
			this.themeName(theme.name);
			this.themeKey(theme.key);
			this.themeColor(theme.color);
			this.themeStyle(theme.style);

			this.background(theme.background);
			this.backgroundImage(theme.background.image);
			this.backgroundImageOpacity(theme.background['image-alpha']);
			this.backgroundImagePosition(theme.background.position);
			this.keyColor(theme.colors.letters);
			this.innerBtnColor(theme.colors.inner_btn);
			this.homerowColor(theme.colors.homerow);
			this.candyColor(theme.colors.candy);
			this.candyFocusColor(theme.colors.candyfocus);
			this.pressBtnColor(theme.colors.press_btn);
			this.tileColor(theme.colors.tile);

			if ( typeof theme.icon !== 'undefined' ) {
				this.themeIcon(theme.icon.icon_class);
				this.themeIconColor(theme.icon.icon_color);
			}

			if ( theme.icons && theme.icons.tile ) {
				this.tileIcon(theme.icons.tile);
			}

			typeof ( theme.colors.shade_btn === 'undefined' ) ? this.shadeBtnColor('#00FFFFFF') : this.shadeBtnColor(theme.colors.shade_btn);

			typeof theme.colors.outline === 'undefined' ? this.outlineColor('#00FFFFFF') : this.outlineColor(theme.colors.outline);
		}

		return this.options.theme;
	};

	Fleksy.prototype.themeName = function(name) {
		if ( name ) {
			this.options.theme.name = name;
		}

		return this.options.theme.name;
	};

	Fleksy.prototype.themeKey = function(key) {
		if ( key ) {
			this.options.theme.key = key;
		}

		return this.options.theme.key;
	};

	Fleksy.prototype.themeColor = function(color) {
		if ( typeof color !== 'undefined' ) {
			this.options.theme.color = color;
		} else {
			delete this.options.theme.color;
		}
	};

	Fleksy.prototype.themeStyle = function(style) {
		if ( typeof style !== 'undefined' ) {
			this.options.theme.style = style;
		} else {
			delete this.options.theme.style;
		}
	};

	Fleksy.prototype.themeSpecial = function(array) {
		if ( Array.isArray(array) ) {
			this.options.theme.special = array;
		} else {
			delete this.options.theme.special;
		}
	};

	Fleksy.prototype.themeIcon = function(icon) {
		if ( typeof icon !== 'undefined' ) {
			this.options.theme.icon.icon_class = icon;
		} else {
			delete this.options.theme.icon.icon_class;
		}
	};

	Fleksy.prototype.themeIconColor = function(color) {
		if ( typeof color !== 'undefined' ) {
			this.options.theme.icon.icon_color = color;
		} else {
			delete this.options.theme.icon.icon_color;
		}
	};

	Fleksy.prototype.background = function(background) {
		var backgroundColors = background && background.gradient ? background.gradient : null;

		if ( backgroundColors !== null ) {
			if ( backgroundColors.length > 1 ) {
				$(this.options.element).css({
					"background-image": "-webkit-linear-gradient(top, " +
						backgroundColors.map( function(color) { return tinycolor(color).toRgbString() }).join(',')
					+ ")"
				});
			} else {
				$(this.options.element).css({
					'background-color': tinycolor(backgroundColors[0]).toRgbString(),
					'background-image': 'none'
				});
			}
		}
	};

	Fleksy.prototype.backgroundImage = function(image) {
		if ( image ) {
			this.options.theme.background.image = image;

			if ( typeof this.options.theme.background.position === 'undefined' ) {
				this.backgroundImagePosition('scale');
			}

			$(this.options.element).find('.kb-inner').css({
				"background-image": 'url('+this.options.imagePath+image+')'
			});
		} else {
			delete this.options.theme.background.image;
			delete this.options.theme.background['image-alpha'];

			$(this.options.element).find('.kb-inner').css({
				"background-image": "none"
			});
		}

		//return this.options.theme.background.image; TODO maybe figure out a way to return this
	};

	Fleksy.prototype.backgroundImagePreview = function(image) {
		if ( image ) {
			$(this.options.element).find('.kb-inner').css({
				"background-image": 'url('+this.options.imagePath+image+')'
			});
		}
	};

	Fleksy.prototype.backgroundImageOpacity = function(opacity) {
		if ( opacity ) {
			this.options.theme.background['image-alpha'] = opacity;
			$(this.options.element).find('.kb-inner').css({
				"opacity": opacity
			});
		}

		return this.options.theme.background['image-alpha'];
	};

	Fleksy.prototype.backgroundImagePosition = function(position) {
		if ( position ) {
			this.options.theme.background.position = position;

			if ( position === 'scale' ) {
				$(this.options.element).find('.kb-inner').css({
					"backgroundSize": 'cover'
				});
			} else if ( position === 'center' ) {
				$(this.options.element).find('.kb-inner').css({
					"backgroundSize": '60%',
					'backgroundPosition': 'center center'
				});
			} else {
				$(this.options.element).find('.kb-inner').css({
					"backgroundSize": 'auto'
				});
			}
		}

		return this.options.theme.background.position;
	};

	Fleksy.prototype.backgroundGradient = function(gradient) {
		//TODO fill this in from this.background
	};

	Fleksy.prototype.backgroundColor = function(color, idx) {
		if ( color !== null && typeof idx === 'number' ) {
			this.options.theme.background.gradient[idx] ? this.options.theme.background.gradient[idx] = color : this.options.theme.background.gradient.push(color);
			this.background(this.options.theme.background);
		} else if ( typeof idx === 'number' ) {
			this.options.theme.background.gradient.splice(idx, 1);
			this.background(this.options.theme.background);
		}

		return this.options.theme.background.gradient;
	};

	Fleksy.prototype.clearBackground = function() {
		this.options.theme.background.gradient = [];
	};

	Fleksy.prototype.keyColor = function(color) {
		if ( color ) {
			$(this.options.element).css({
				'color': tinycolor(color).toRgbString()
			});
			this.options.theme.colors.letters = color;
		}

		return this.options.theme.colors.letters;
	};

	Fleksy.prototype.homerowColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-spacebar, .fl-row-mid').css({
				'background-color': tinycolor(color).toRgbString()
			});
			this.options.theme.colors.homerow = color;
		}

		return this.options.theme.colors.homerow;
	};

	Fleksy.prototype.innerBtnColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-btn').not('.fl-spacebar').css({
				'background-color': tinycolor(color).toRgbString()
			});
			this.options.theme.colors.inner_btn = color;
		}

		return this.options.theme.colors.inner_btn;
	};

	Fleksy.prototype.shadeBtnColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-btn').css({
				'border-bottom': '0.05em solid ' + tinycolor(color).toRgbString()
			});
			this.options.theme.colors.shade_btn = color;
		}

		return this.options.theme.colors.shade_btn;
	};

	Fleksy.prototype.pressBtnColor = function(color) {
		if ( color ) {
			var that = this;
			$(this.options.element).find('.fl-btn').on('mousedown', function(e) {
				$(this).css({
					'background': tinycolor(color).toRgbString()
				});
			}).on('mouseup', function(e) {
				var originalColor = $(this).hasClass('fl-spacebar') ? that.options.theme.colors.homerow : that.options.theme.colors.inner_btn;

				$(this).css({
					'background': tinycolor(originalColor).toRgbString()
				});
			});

			this.options.theme.colors.press_btn = color;
		}

		return this.options.theme.colors.press_btn;
	};

	Fleksy.prototype.tileColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-key-pop i').css({
				'color': tinycolor(color).toRgbString()
			});

			this.options.theme.colors.tile = color;
		}

		return this.options.theme.colors.tile;
	};

	Fleksy.prototype.tileIcon = function(icon) {
		if ( icon ) {
			$(this.options.element).find('.fl-key-pop i').text(icon);
			this.options.theme.icons.tile = icon;
		}

		return this.options.theme.icons.tile;
	};

	Fleksy.prototype.candyColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-candy').css({
				'color': tinycolor(color).toRgbString()
			});
			this.options.theme.colors.candy = color;
		}

		return this.options.theme.colors.candy;
	};

	Fleksy.prototype.candyFocusColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-candyfocus').css({
				'color': tinycolor(color).toRgbString()
			});
			this.options.theme.colors.candyfocus = color;
		}

		return this.options.theme.colors.candyfocus;
	};

	Fleksy.prototype.outlineColor = function(color) {
		if ( color ) {
			$(this.options.element).find('.fl-key').css({
				'text-shadow': '0 0 0.06em ' + tinycolor(color).toRgbString()
			});

			this.options.theme.colors.outline = color;
		}

		return this.options.theme.colors.outline;
	};

	Fleksy.prototype.element = function() {
		return this.options.element;
	};

	Fleksy.prototype.togglePop = function(toggle) {
		var $row = $(this.options.element).find('.fl-row').eq(1),
				$keyA = $row.find('.fl-key').eq(1),
				$keyB = $row.find('.fl-key').eq(2),
				$keyC = $row.find('.fl-key').eq(3);

		if ( toggle ) {
			$keyA.find('span').css('opacity', 0);
			$keyA.find('b').css({
				'top': '-10%',
				'opacity': 0.7
			}).addClass('fl-toggled');

			$keyB.find('span').css('opacity', 0);
			$keyB.find('b').css({
				'top': '-18%',
				'opacity': 1
			}).addClass('fl-toggled');

			$keyC.find('span').css('opacity', 0);
			$keyC.find('b').css({
				'top': '-1%',
				'opacity': 0.4
			}).addClass('fl-toggled');
		} else {
			$keyA.find('span').css('opacity', 1);
			$keyA.find('b').css({
				'top': 0,
				'opacity': 0
			}).removeClass('fl-toggled');

			$keyB.find('span').css('opacity', 1);
			$keyB.find('b').css({
				'top': 0,
				'opacity': 0
			}).removeClass('fl-toggled');

			$keyC.find('span').css('opacity', 1);
			$keyC.find('b').css({
				'top': 0,
				'opacity': 0
			}).removeClass('fl-toggled');
		}
	};

	function fitToViewport() {
		var $keyboard = $(this.options.element),
				domKeyboardWidth = $keyboard.width();

		if ( domKeyboardWidth !== this.width() ) {
			this.width(domKeyboardWidth);
		}
	}

	return Fleksy;
}));
