var BannerActions = {

	/**
	 * Scrolls to a specific element
	 *
	 * @param {String|Object} el Element selector, jQuery or not
	 * @param {Number} [offset] Top offset from element
	 * @param {Function} [callback] A callback function, runs after scroll has finished
	 */
	scrollToItem: function(el, offset, callback) {
		if ( typeof el == 'string' && !(el instanceof jQuery) ) {
			el = $(el);
		}

		offset = offset || 0;

		if ( el.length ) {
			var distance = el.offset().top + offset;

			$('body').animate({
				scrollTop: distance
			}, 0, function() {
				FleksyAPI.scrollTo(distance*window.devicePixelRatio);
				if ( callback ) callback();
			});
		}
	},

	/**
	 * Scrolls to a theme and selects it
	 *
	 * @param {String} themeKey The theme unique ID (key)
	 */
	selectTheme: function(themeKey) {
		BannerActions.scrollToItem('.'+themeKey.replace(/\s+/g, ''), -10, function() {
			setTimeout(FleksyAPI.selectTheme.bind(null, themeKey), 150);
		});
	},

	/**
	* Launches the ThemeBuilder
	*/
	loadThemeBuilder: function() {
		FleksyAPI.loadThemeBuilder();
	},

	/**
	* Displays a toast
	* @param {String} toast The message to be displayed
	*/
	showToast: function(toast) {
		FleksyAPI.showToast(toast);
	},

	/**
	* Opens up the keyboard and searches the keyword in highlights
	* @param {String} keyword the search term
	*/
	searchHighlights: function(keyword) {
		FleksyAPI.searchHighlights(keyword);
	},

    coinsEarn: function() {
        FleksyAPI.coinsEarn();
    },

    coinsHelp: function() {
        FleksyAPI.coinsHelp();
    },

    purchaseFleksyCoinsBox: function(id) {
        FleksyAPI.purchaseFleksyCoinsBox(id);
    }
};
