var FleksyAPI,
		sizeData,
		spacebarData,
		sizesData,
		i18nData,
		count = 0;

function setPlatform( name ) {
	if ( name == "Android" ) {
		FleksyAPI = new FleksyAPIAndroid();
		FleksyAPI.requestSizesData();
	} else if ( name == 'iOS' ) {
		FleksyAPI = new FleksyAPIiOS();
		FleksyAPI.requestSizesData();
	} else {
		//console.log("Undefined Platform!");
	}
}

function selectedThemeCallback( key, result ) { }
function onDestroyCallback() { }

function stringsCallback( strings ) {
	if ( typeof strings === 'string' ) {
		i18nData = $.parseJSON( strings );
	} else {
		i18nData = strings;
	}

	if ( count > 1 ) {
		FLSizes.translateStrings(i18nData);
	} else {
		_.delay(stringsCallback, 250, strings);
	}
}

function selectedKeyboardSizeCallback(size, result) {
	sizeData = size;
	count++;
	//console.log('selectedKeyboardSizeCallback');
}

function enabledSpacebarCallback(isEnabled, result) {
	spacebarData = isEnabled;
	count++;

	if ( typeof FLSizes !== 'undefined' ) {
		FLSizes.setModeState();
	}

	//console.log('enabledSpacebarCallback');
}

function sizesCallback( sizes ) {
	if ( typeof sizes == 'string' ) {
		sizesData = $.parseJSON( sizes );
	} else {
		sizesData = sizes;
	}

	//console.log('sizesCallback');

	if ( count > 1 ) {
		FLSizes.init();
	} else {
		_.delay(sizesCallback, 500, sizes);
	}
}

var s,
FLSizes = {
	settings: {
		size: null,
		spacebar: null,
		sizes: null,
		i18nData: null
	},

	/**
	 * Main init method.
	 */
	init: function() {
		//console.log('init');

		s = this.settings;
		s.size = sizeData;
		s.spacebar = spacebarData;
		s.sizes = sizesData.sizes;
		s.i18nStrings = i18nData || null;

		_.each(s.sizes, function(key) {
			FLSizes.makeSize(key).appendTo('.sizes');
		});

		FLSizes.bindUI();
		FLSizes.setSizesState();
		FLSizes.setModeState();
		FLSizes.bindSizeUI();
		FLSizes.bindModeUI();
	},

	/**
	 * Sets user preferences for the sizes
	 */
	setSizesState: function() {
		s.size = sizeData;
		$("#size-slide").val($('#' + s.size).parent().index());
	},

	/**
	 * Sets user preferences for the modes
	 */
	setModeState: function() {
		if ( typeof s !== 'undefined' ) {
			s.spacebar = spacebarData;
			$("#mode-slide").val( s.spacebar == "true" ? 0 : 1 );
		}
	},

	makeSize: function(key) {
		var size = $(".template").clone().removeClass('template'),
				defaultLabel = key[0].toUpperCase() + key.substring(1),
				label = s.i18nStrings !== null ? s.i18nStrings.size[defaultLabel] : defaultLabel;

		size.find('a').attr('id', key);
		size.find('.text').text(label);

		return size;
	},

	/**
	 * Creates sliders and binds the general UI.
	 */
	bindUI: function() {
		var $selection = $('.selection');

		$('.slide').each(function() {
			var $this = $(this);
			$this.noUiSlider($this.data('options'));
		});

		$('.noUi-handle').on('touchstart', function() {
			FleksyAPI.holdHorizontalSlide();
		});

		$('.noUi-handle').on('touchend', function() {
			FleksyAPI.resumeHorizontalSlide();
		});


		$selection.on('touchstart', function(e) {
			$(this).addClass('highlight');
		});

		$selection.on('touchend', function(e) {
			$(this).removeClass('highlight');
		});

		$(document).on('touchcancel', function() {
			FleksyAPI.dismissKeyboard();
		});

		FastClick.attach(document.body);
	},

	/**
	 * Binds tapping events on the size selections.
	 */
	bindSizeUI: function() {
		var $selection = $('.sizes').find('a');

		$selection.on('click', function(e) {
			var $this = $(this);
			$this.parents('.wrap').find('.slide').val($this.parent().index());
			e.preventDefault();
		});

		$("#size-slide").on({
			set: function(data, val) {
				val = parseInt(val);
				var slider = $(data.target),
						sizes = slider.parent().find('.selections');

				FleksyAPI.selectKeyboardSize(sizes.find('li').eq(val).find('a').attr('id'));
			}
		});
	},

	/**
	 * Binds tapping events on the mode selections.
	 */
	bindModeUI: function() {
		var $selection = $('.modes').find('a');

		$selection.on('click', function(e) {
			var $this = $(this);
			FleksyAPI.selectSpacebarState( !parseInt($this.parent().index()) );
			e.preventDefault();
		});
	},

	translateStrings: function( translations ) {
		$('[data-i18n]').each(function() {
			var that = $(this),
					pointer = that.data('i18n').split('.');

			that.html( translations[pointer[0]][pointer[1]] );
		});
	}
};

(function($) {
	$('.refresh').on('click', function(e) {
		location.reload();

		e.preventDefault();
	});
})(jQuery);
